
#include "Arduino.h"
#define PHASE_A 11
#define PHASE_B 12
#define PHASE_C 13
#define NECHANT 128
#define SHIFT_ACCUM 25
#define GET_DATA 10
#define SET_DATA 11
#define DATA_0_SIZE 4 // fréquence, flottant 32 bits
#define DATA_1_SIZE 4 // amplitude, flottant 32 bits
uint8_t data_0[DATA_0_SIZE];
uint8_t data_1[DATA_1_SIZE];
bool data_0_ready = true;
bool data_0_request = false;
bool data_1_ready = true;
bool data_1_request = false;
float frequence = 1; // data_0
float amp = 0.5;
uint32_t period_pwm = 1000; // en microsecondes (10 kHz)

uint32_t icr;
uint32_t table_onde[NECHANT];
uint32_t indexA,indexB,indexC;
uint32_t accum1,accum2,accum3,increm;
uint16_t diviseur[6] = {0,1,8,64,256,1024};            
                 
void init_pwm_timer1(uint32_t period) {
    char clockBits;
    TCCR1A = 0;
    TCCR1A |= (1 << COM1A1); //Clear OCnA/OCnB/OCnC on compare match, set OCnA/OCnB/OCnC at BOTTOM (non-inverting mode)
    TCCR1A |= (1 << COM1B1);
    TCCR1A |= (1 << COM1C1);
    TCCR1B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR1
    int d = 1;
    icr = (F_CPU/1000000*period/2);
    while ((icr>0xFFFF)&&(d<6)) { // choix du diviseur d'horloge
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
   clockBits = d;
   ICR1 = icr; // valeur maximale du compteur
   TIMSK1 = 1 << TOIE1; // overflow interrupt enable
   sei(); // activation des interruptions
   TCNT1 = 0; // mise à zéro du compteur
   TCCR1B |= clockBits; // déclenchement du compteur
}
             
                 
ISR(TIMER1_OVF_vect) { // Timer 1 Overflow interrupt
  accum1 += increm;
  accum2 += increm;
  accum3 += increm;
  indexA = accum1 >> SHIFT_ACCUM;
  OCR1A = table_onde[indexA];
  indexB = accum2 >> SHIFT_ACCUM;
  OCR1B = table_onde[indexB];
  indexC = accum3 >> SHIFT_ACCUM;
  OCR1C = table_onde[indexC];
}
                 
void set_sinus_table() {
  int i;
  float dt = 2*3.1415926/NECHANT;
  for(i=0; i<NECHANT; i++) {
    table_onde[i] = icr*(amp*sin(i*dt)+amp)*0.5;
  }  
}           
                 
void set_frequence() {
  increm = (uint32_t) (((float)(0xFFFFFFFF))*((float)(frequence)*1e-6*(float)(period_pwm))); // incrément de l'accumulateur de phase
}            
                 
void setup() {
  Serial.begin(115200);
  Serial.setTimeout(0);
  char c = 0;
  Serial.write(c);
  c = 255;
  Serial.write(c);
  c = 0;
  Serial.write(c);
  pinMode(PHASE_A,OUTPUT);
  pinMode(PHASE_B,OUTPUT);
  pinMode(PHASE_C,OUTPUT);
  pinMode(ENA,OUTPUT);
  digitalWrite(ENA,HIGH);
  pinMode(ENB,OUTPUT);
  digitalWrite(ENB,HIGH);
  pinMode(7,OUTPUT);
  digitalWrite(7,LOW);
  init_pwm_timer1(period_pwm);
  set_sinus_table();
  accum1 = 0;
  accum2 = ((uint32_t)(NECHANT * 0.3333333)) << SHIFT_ACCUM;
  accum3 = ((uint32_t)(NECHANT * 0.6666666)) << SHIFT_ACCUM;
  set_frequence();

}
        
void set_data() {
  char n;
  while (Serial.available()<1) {};
  n = Serial.read();
  if (n==0) {
    while (Serial.available()<DATA_0_SIZE) {};
    Serial.readBytes(data_0,DATA_0_SIZE);
    memcpy(&frequence,data_0,DATA_0_SIZE);
    set_frequence();
  }
  else if (n==1) {
    while (Serial.available()<DATA_1_SIZE) {};
    Serial.readBytes(data_1,DATA_1_SIZE);
    memcpy(&amp,data_1,DATA_1_SIZE);
    set_sinus_table();
  }
}                
                
void get_data() {
  char n;
  while (Serial.available()<1) {};
  n = Serial.read();
  if (n==0) {
    data_0_request = true;
    memcpy(data_0,&frequence,DATA_0_SIZE);
  }
  if (n==1) {
    data_1_request = true;
    memcpy(data_1,&amp,DATA_1_SIZE);
  }
}
void send_data() {
  if ((data_0_ready)&&(data_0_request)) {
      data_0_ready = false;
      data_0_request = false;
      Serial.write(data_0,DATA_0_SIZE);
  }
  if ((data_1_ready)&&(data_1_request)) {
      data_1_ready = false;
      data_1_request = false;
      Serial.write(data_1,DATA_1_SIZE);
  }
}
               
                
void read_serial() {
   char com;
   if (Serial.available()>0) {
        com = Serial.read();
        if (com==GET_DATA) get_data();
        else if (com==SET_DATA) set_data();
   }
}	           
                
void loop() {
  read_serial();
  send_data();
}             
                  